/* 
 * Copyright (C) 2004 Romain Dolbeau <romain@dolbeau.org>
 *
 *  This file is subject to the terms and conditions of the GNU General Public
 *  License. See the file COPYING in the main directory of this archive for
 *  more details.
 */

package org.dolbeau.mencgen;

import org.dolbeau.xsCOEj.Results;

public class lavcResults extends Results {
    private static java.text.NumberFormat format = new java.text.DecimalFormat();
    static {
        format.setMinimumFractionDigits(3);
        format.setMaximumFractionDigits(3);
    }

    double psnrY, psnrCb, psnrCr, psnrAll;

    public String toString() {
        return "PSNR: " +
            "Y=" + format.format(psnrY) + ", " +
            "Cb=" +  format.format(psnrCb) + ", " +
            "Cr=" +  format.format(psnrCr) + ", " +
            "All=" +  format.format(psnrAll) + " " +
            super.toString();
    }

    public String toHTMLString() {
        return "PSNR: " +
            "Y=" +  format.format(psnrY) + ", " +
            "Cb=" +  format.format(psnrCb) + ", " +
            "Cr=" +  format.format(psnrCr) + ", " +
            "<font color=#7f0000>All=" +  format.format(psnrAll) + "</font> " +
            super.toHTMLString();
    }
    
    public double getMetricAsADouble() {
        return psnrAll;
    }

    public boolean isBiggerBetter() {
        return true;
    }

    public boolean isWithinPercent(Results r, double p) {
        lavcResults r2 = (lavcResults) r;
        return (psnrAll >= (r2.psnrAll * (1. - (p / 100.))));
    }

    public lavcResults(double y, double cb, double cr, double all, double s) {
        super(s);
        psnrY = y;
        psnrCb = cb;
        psnrCr = cr;
        psnrAll = all;
    }

    public lavcResults(lavcResults l, double s) {
        super(s);
        if (l != null) {
            psnrY = l.psnrY;
            psnrCb = l.psnrCb;
            psnrCr = l.psnrCr;
            psnrAll = l.psnrAll;
        } else {
            psnrY = -1.;
            psnrCb = -1.;
            psnrCr = -1.;
            psnrAll = -1.;
        }
    }

    public boolean isValid() {
        return
            ((psnrY > -1.) &&
             (psnrCb > -1.) &&
             (psnrCr > -1.) &&
             (psnrAll > -1.) &&
             super.isValid());
    }

    public int compareTo(Object o) {
        lavcResults o2 = (lavcResults)o;

        if (psnrAll < o2.psnrAll)
            return -1;
        if (psnrAll > o2.psnrAll)
            return 1;
        if (psnrY < o2.psnrY)
            return -1;
        if (psnrY > o2.psnrY)
            return 1;
        if ((psnrCb + psnrCr) < (o2.psnrCb + o2.psnrCr))
            return -1;
        if ((psnrCb + psnrCr) > (o2.psnrCb + o2.psnrCr))
            return 1;
        return super.compareTo(o);
    }
}
