/* 
 * Copyright (C) 2004 Romain Dolbeau <romain@dolbeau.org>
 *
 *  This file is subject to the terms and conditions of the GNU General Public
 *  License. See the file COPYING in the main directory of this archive for
 *  more details.
 */

package org.dolbeau.xsCOEj;

import java.util.*;

public class OptionInstanceList extends ArrayList implements Comparable {
    public final static int MAX_ATTEMPTS = 20;
    
    public OptionInstanceList() {
        super();
    }
    
    public OptionInstanceList(OptionInstanceList l) {
        super(l);
    }

    public boolean add(OptionInstance oi) {
        if (oi == null) {
            throw new IllegalArgumentException("Only non-null object can be added to an OptionInstanceList");
        }
        return super.add(oi);
    }

    public boolean add(Object o) {
        if (o == null) {
            throw new IllegalArgumentException("Only non-null object can be added to an OptionInstanceList");
        }
        if (!(o instanceof OptionInstance)) {
            throw new IllegalArgumentException("Only OptionInstance object can be added to an OptionInstanceList");
        }
        return super.add(o);
    }

    public int compareTo(Object o) {
        OptionInstanceList o2 = (OptionInstanceList)o;

        if (size() < o2.size())
            return -1;
        if (size() > o2.size())
            return 1;
        
        Iterator it1 = iterator();
        /*
        Iterator it2 = o2.iterator();
        while (it1.hasNext() && it2.hasNext()) {
            OptionInstance i1 = (OptionInstance)it1.next();
            OptionInstance i2 = (OptionInstance)it2.next();
            int c = i1.compareTo(i2);
            if (c != 0)
                return c;
        }
        */
        while (it1.hasNext()) {
            OptionInstance i1 = (OptionInstance)it1.next();
            OptionInstance i2 = o2.getOptionInstance(i1.getParentOption());
            int c = i1.compareTo(i2);
            if (c != 0)
                return c;
        }

        return 0;
    }

    /** Get the OptionInstance in the list that belongs to the given Option
     * @param opt The Option whose OptionInstance is to be returned
     * @return The OptionInstance in this list, representing the given Option
     */
    public OptionInstance getOptionInstance(Option opt) {
        List allInstances = opt.getAllInstances();
        Iterator it = allInstances.iterator();
        while (it.hasNext()) {
            OptionInstance loi = (OptionInstance)it.next();
            if (contains(loi))
                return loi;
        }
        return null; // oups
    }

    public static OptionInstanceList buildRandomList(Set allOptions, Random random) {
        OptionInstanceList temp = new OptionInstanceList();
        Iterator it = allOptions.iterator();
        while (it.hasNext()) {
            Option opt = (Option)it.next();
            List allInstances = opt.getAllInstances();
            OptionInstance loi = (OptionInstance)allInstances.get(random.nextInt(allInstances.size()));
            if (temp.isCompatible(loi)) {
                temp.add(loi);
            }
            else { // try again, non-random
                if (temp.isCompatible(opt.getDefaultInstance())) {
                    temp.add(opt.getDefaultInstance());
                } else {
                    return null;
                }
            }
        }
        Collections.sort(temp);
        return temp;
    }
    
    public static OptionInstanceList buildDefaultList(Set allOptions) {
        OptionInstanceList temp = new OptionInstanceList();
        Iterator it = allOptions.iterator();
        while (it.hasNext()) {
            Option opt = (Option)it.next();
            OptionInstance loi = opt.getDefaultInstance();
            temp.add(loi);
        }
        Collections.sort(temp);
        return temp;
    }

    public boolean isCompatible(OptionInstance oi) {
        Iterator it = iterator();
        while (it.hasNext()) {
            if (!oi.isCompatible((OptionInstance)it.next()))
                return false;
        }
        return true;
    }

    public String toPrettyString(String sep) {
        String temp = null;
        Iterator it = iterator();
        while (it.hasNext()) {
            OptionInstance o = (OptionInstance)it.next();

            if ((o.getOptionString() != null) &&
                (!o.getParentOption().isVirtual())) {
                if (temp == null)
                    temp = o.getOptionString();
                else
                    temp = temp + sep + o.getOptionString();
            }
        }
        return temp;
    }

    public OptionInstanceList updateToNewOptionSet(Set newAllOptions) {
        OptionInstanceList updated = new OptionInstanceList();
        Iterator it = iterator();
        while (it.hasNext() && (updated != null)) {
            OptionInstance oi = (OptionInstance)it.next();
            
            
            Iterator it2 = newAllOptions.iterator();
            while (it2.hasNext() && (updated != null)) {
                Option o = (Option)it2.next();
                if (o.equals(oi.getParentOption())) {
                    if (!o.hasInstance(oi)) {
                        if (updated.isCompatible(o.getDefaultInstance()) &&
                            (!o.mustChoose()))
                            updated.add(o.getDefaultInstance());
                        else
                            updated = null;
                    } else {
                        if (updated.isCompatible(oi))
                            updated.add(oi);
                        else
                            updated = null;
                    }
                }
            }
        }
        return updated;
    }
}
