/* 
 * Copyright (C) 2004 Romain Dolbeau <romain@dolbeau.org>
 *
 *  This file is subject to the terms and conditions of the GNU General Public
 *  License. See the file COPYING in the main directory of this archive for
 *  more details.
 */

package org.dolbeau.xsCOEj;

/**
 * The abstract Class representing the result of a command.
 * A metric (with a total order on it) is required in subclasses.
 * @author Romain Dolbeau <romain@dolbeau.org>
 * @version  $Id: Results.java,v 1.1.1.1 2004/12/06 13:52:41 dolbeau Exp $
 * @see ResultsPool
 */
public abstract class Results implements Comparable, java.io.Serializable {
    private double seconds = -1.;

    public abstract boolean isWithinPercent(Results r, double p);

    public double getSeconds() {
        return seconds;
    }

    public abstract double getMetricAsADouble();
    
    public abstract boolean isBiggerBetter();

    public boolean equals(Object o) {
        if (o instanceof Results)
            return compareTo(o) == 0;
        return false;
    }

    public String toString() {
        return " in " + seconds + " seconds";
    }

    public String toHTMLString() {
        return " in <font color=#00007f>" + seconds + "</font> seconds";
    }

    public Results(double s) {
        seconds = s;
    }

    public boolean isValid() {
        return (seconds > -1.);
    }

    public int compareTo(Object o) {
        Results o2 = (Results)o;
        int base = isBiggerBetter() ? 1 : -1;

        if (getMetricAsADouble() > o2.getMetricAsADouble())
            return +base;
        if (getMetricAsADouble() < o2.getMetricAsADouble())
            return -base;
        if (seconds < o2.seconds)
            return 1;
        if (seconds > o2.seconds)
            return -1;
        return 0;
    }
}
